@@ -20,18 +20,17 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef __MAIN_H__
-#define __MAIN_H__
+#ifndef __GTHUMB_INIT_H__
+#define __GTHUMB_INIT_H__
 
 #include "image-viewer.h"
 #include "preferences.h"
 
 extern Preferences       preferences;
-extern PngExporterPref   png_exporter_pref;
 
 
 void  gthumb_init    ();
 void  gthumb_release ();
 
 
-#endif /* __MAIN_H__ */
+#endif /* __GTHUMB_INIT_H__ */
