@@ -21,6 +21,7 @@
  */
 
 #include <config.h>
+#define GDK_PIXBUF_ENABLE_BACKEND
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
@@ -30,12 +31,15 @@
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <gdk-pixbuf/gdk-pixbuf-animation.h>
 #include <sys/stat.h>
 #include "image-loader.h"
 #include "gthumb-marshal.h"
+#include "file-utils.h"
 
 
 #define REFRESH_RATE 5
+G_LOCK_DEFINE_STATIC (pixbuf_loader_lock);
 
 
 struct _ImageLoaderPrivateData {
@@ -262,7 +266,7 @@ image_loader_init (ImageLoader *il)
 GType
 image_loader_get_type ()
 {
-        static guint type = 0;
+        static GType type = 0;
 
         if (! type) {
                 GTypeInfo type_info = {
@@ -380,11 +384,17 @@ image_loader_set_pixbuf (ImageLoader *il,
 
 	priv = il->priv;
 
-	if (priv->pixbuf != NULL)
-		g_object_unref (priv->pixbuf);
+	g_mutex_lock (priv->yes_or_no);
 
-	g_object_ref (pixbuf);
-	priv->pixbuf = pixbuf;
+	if (priv->pixbuf != NULL) {
+		g_object_unref (priv->pixbuf);
+		priv->pixbuf = NULL;
+	}
+
+	if (pixbuf != NULL) 
+		priv->pixbuf = gdk_pixbuf_copy (pixbuf);
+
+	g_mutex_unlock (priv->yes_or_no);
 }
 
 
@@ -409,17 +419,21 @@ image_loader_sync_pixbuf (ImageLoader *il)
 	}
 
 	pixbuf = gdk_pixbuf_animation_get_static_image (priv->animation);
+	g_object_ref (pixbuf);
 
 	if (priv->pixbuf == pixbuf) {
+		g_object_unref (pixbuf);
 		g_mutex_unlock (priv->yes_or_no);
 		return;
 	}
 
-	if (priv->pixbuf != NULL) 
+	if (priv->pixbuf != NULL) {
 		g_object_unref (priv->pixbuf);
+		priv->pixbuf = NULL;
+	}
 	if (pixbuf != NULL) 
-		g_object_ref (pixbuf);
-     	priv->pixbuf = pixbuf;
+		priv->pixbuf = gdk_pixbuf_copy (pixbuf);
+	g_object_unref (pixbuf);
 
 	g_mutex_unlock (priv->yes_or_no);
 }
@@ -453,17 +467,21 @@ image_loader_sync_pixbuf_from_loader (ImageLoader     *il,
 	}
 
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (pb_loader);
+	g_object_ref (pixbuf);
 
 	if (priv->pixbuf == pixbuf) {
+		g_object_unref (priv->pixbuf);
 		g_mutex_unlock (priv->yes_or_no);
 		return;
 	}
 
-	if (priv->pixbuf != NULL) 
+	if (priv->pixbuf != NULL) {
 		g_object_unref (priv->pixbuf);
-	if (pixbuf != NULL) 
-		g_object_ref (pixbuf);
-     	priv->pixbuf = pixbuf;
+		priv->pixbuf = NULL;
+	}
+	if (pixbuf != NULL)
+		priv->pixbuf = gdk_pixbuf_copy (pixbuf);
+	g_object_unref (pixbuf);
 
 	g_mutex_unlock (priv->yes_or_no);
 }
@@ -516,7 +534,7 @@ load_image_thread (void *thread_data)
 {
 	ImageLoader            *il = thread_data;
 	ImageLoaderPrivateData *priv = il->priv;
-	GdkPixbufAnimation     *animation;
+	GdkPixbufAnimation     *animation = NULL;
 	GError                 *error = NULL;
 
 	for (;;) {
@@ -540,14 +558,28 @@ load_image_thread (void *thread_data)
 
 		g_mutex_lock (priv->yes_or_no);
 
+		G_LOCK (pixbuf_loader_lock);
+
+		animation = NULL;
 		if (path != NULL) {
-			if (priv->loader != NULL)
+			if (priv->loader != NULL) 
 				animation = (*priv->loader) (path, &error, priv->loader_data);
-			else
-				animation = gdk_pixbuf_animation_new_from_file (path, &error);
-		} else 
-			animation = NULL;
+			else {
+				if (image_is_gif (path))
+					animation = gdk_pixbuf_animation_new_from_file (path, &error);
+				else {
+					GdkPixbuf *pixbuf;
+					pixbuf = gdk_pixbuf_new_from_file (path, &error);
+					if (pixbuf != NULL) {
+						animation = gdk_pixbuf_non_anim_new (pixbuf);
+						g_object_unref (pixbuf);
+					}
+				}
+			}
+		} 
 		
+		G_UNLOCK (pixbuf_loader_lock);
+
 		priv->loader_done = TRUE;
 
 		if (priv->animation != NULL)
@@ -1032,7 +1064,7 @@ image_loader_load_from_image_loader (ImageLoader *to,
 		to->priv->animation = NULL;
 	}
 
-	if (from->priv->animation) {
+	if (from->priv->animation) { /* FIXME*/
 		g_object_ref (from->priv->animation);
 		to->priv->animation = from->priv->animation;
 	}
