@@ -53,6 +53,7 @@
 #include "dlg-scale-image.h"
 #include "dlg-crop.h"
 #include "dlg-write-to-cd.h"
+#include "dlg-image-prop.h"
 #include "fullscreen.h"
 #include "gconf-utils.h"
 #include "gth-pixbuf-op.h"
@@ -2721,7 +2722,7 @@ activate_action_help_about (GtkAction *action,
 		return;
 	}
 
-	logo = gdk_pixbuf_new_from_file (PIXMAPSDIR "/gthumb.png", NULL);
+	logo = gdk_pixbuf_new_from_file (PIXMAPSDIR "/gthumb-logo.png", NULL);
 	about = gnome_about_new (_("gThumb"), 
 				 VERSION,
 				 "Copyright \xc2\xa9 2001-2004 Free Software Foundation, Inc.",
@@ -2825,10 +2826,7 @@ set_action_sensitive (GThumbWindow *window,
 {
 	GtkAction *action;
 	action = gtk_action_group_get_action (window->actions, action_name);
-	if (action == NULL) 
-		g_print ("%s NON ESISTE\n", action_name);
-	else
-		g_object_set (action, "sensitive", sensitive, NULL);
+	g_object_set (action, "sensitive", sensitive, NULL);
 }
 
 
