@@ -24,7 +24,7 @@
 #define DLG_CATALOG_H
 
 #include <glib.h>
-#include "window.h"
+#include "gthumb-window.h"
 
 void   dlg_add_to_catalog                   (GThumbWindow *window,
 					     GList *list);
