@@ -32,6 +32,7 @@
 #include <glade/glade.h>
 
 #include "typedefs.h"
+#include "preferences.h"
 #include "main.h"
 #include "glib-utils.h"
 #include "gthumb-window.h"
@@ -40,6 +41,7 @@
 #include "comments.h"
 #include "gth-exif-utils.h"
 #include "dlg-comment.h"
+#include "dlg-image-prop.h"
 
 
 enum {
@@ -54,8 +56,10 @@ enum {
 };
 
 
+#define DIALOG_NAME "comment"
 #define GLADE_FILE "gthumb_comments.glade"
 
+
 typedef struct {
 	GThumbWindow  *window;
 	GladeXML      *gui;
@@ -105,6 +109,15 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
+static gboolean
+unrealize_cb (GtkWidget  *widget, 
+	  DialogData *data)
+{
+	pref_util_save_window_geometry (GTK_WINDOW (widget), DIALOG_NAME);
+	return FALSE;
+}
+
+
 static int
 strcmp_null_tollerant (const char *s1, const char *s2)
 {
@@ -380,7 +393,7 @@ dlg_comment_new (GThumbWindow *window)
 
 	if (window->comment_dlg != NULL) {
 		gtk_window_present (GTK_WINDOW (window->comment_dlg));
-		return;
+		return window->comment_dlg;
 	}
 
 	data = g_new0 (DialogData, 1);
@@ -390,7 +403,7 @@ dlg_comment_new (GThumbWindow *window)
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE , NULL, NULL);
         if (!data->gui) {
                 g_warning ("Could not find " GLADE_FILE "\n");
-                return;
+                return NULL;
         }
 
 	/* Get the widgets. */
@@ -416,6 +429,10 @@ dlg_comment_new (GThumbWindow *window)
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
+	g_signal_connect (G_OBJECT (data->dialog), 
+			  "unrealize",
+			  G_CALLBACK (unrealize_cb),
+			  data);
 	g_signal_connect (G_OBJECT (data->save_button), 
 			  "clicked",
 			  G_CALLBACK (save_clicked_cb),
@@ -456,7 +473,7 @@ dlg_comment_new (GThumbWindow *window)
 	}
 
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
-	gtk_widget_show (data->dialog);
+	pref_util_restore_window_geometry (GTK_WINDOW (data->dialog), DIALOG_NAME);
 	dlg_comment_update (data->dialog);
 
 	return data->dialog;
