@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include <config.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <string.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
@@ -33,6 +34,7 @@
 #include <exif-data.h>
 #include <exif-content.h>
 #include <exif-entry.h>
+#include <exif-utils.h>
 #include "jpegutils/jpeg-data.h"
 #endif /* HAVE_LIBEXIF */
 
@@ -46,6 +48,7 @@
 #include "jpegutils/jpegtran.h"
 #include "pixbuf-utils.h"
 #include "gthumb-stock.h"
+#include "gth-exif-utils.h"
 
 
 #define ROTATE_GLADE_FILE "gthumb_tools.glade"
@@ -68,10 +71,12 @@ typedef struct {
 	GladeXML     *gui;
 
 	GtkWidget    *dialog;
-	GtkWidget    *j_button_box;	
+	GtkWidget    *j_button_box;
+	GtkWidget    *j_button_vbox;
 	GtkWidget    *j_revert_button;;
 	GtkWidget    *j_apply_to_all_checkbutton;
 	GtkWidget    *j_preview_image;
+	GtkWidget    *j_from_exif_checkbutton;
 
 	int           rot_type;    /* 0, 90 ,180, 270 */
 	int           tran_type;   /* mirror, flip */
@@ -79,6 +84,7 @@ typedef struct {
 	GList        *files_changed_list;
 	GList        *current_image;
 	ImageLoader  *loader;
+	GdkPixbuf    *original_preview;
 } DialogData;
 
 
@@ -139,6 +145,117 @@ _gdk_pixbuf_scale_keep_aspect_ratio (GdkPixbuf *pixbuf,
 }
 
 
+static void
+update_rotation_from_exif_data (DialogData *data,
+				GList      *current_image)
+{
+#ifdef HAVE_LIBEXIF
+	FileData           *fd = current_image->data;
+	GthExifOrientation  orientation = get_exif_tag_short (fd->path, EXIF_TAG_ORIENTATION);
+
+	switch (orientation) {
+	case GTH_EXIF_ORIENTATION_TOP_LEFT:
+		data->rot_type = TRAN_ROTATE_0;
+		data->tran_type = TRAN_NONE;
+		break;
+	case GTH_EXIF_ORIENTATION_TOP_RIGHT:
+		data->rot_type = TRAN_ROTATE_0;
+		data->tran_type = TRAN_MIRROR;
+		break;
+	case GTH_EXIF_ORIENTATION_BOTTOM_RIGHT:
+		data->rot_type = TRAN_ROTATE_180;
+		data->tran_type = TRAN_NONE;
+		break;
+	case GTH_EXIF_ORIENTATION_BOTTOM_LEFT:
+		data->rot_type = TRAN_ROTATE_180;
+		data->tran_type = TRAN_MIRROR;
+		break;
+	case GTH_EXIF_ORIENTATION_LEFT_TOP:
+		data->rot_type = TRAN_ROTATE_90;
+		data->tran_type = TRAN_MIRROR;
+		break;
+	case GTH_EXIF_ORIENTATION_RIGHT_TOP:
+		data->rot_type = TRAN_ROTATE_90;
+		data->tran_type = TRAN_NONE;
+		break;
+	case GTH_EXIF_ORIENTATION_RIGHT_BOTTOM:
+		data->rot_type = TRAN_ROTATE_90;
+		data->tran_type = TRAN_FLIP;
+		break;
+	case GTH_EXIF_ORIENTATION_LEFT_BOTTOM:
+		data->rot_type = TRAN_ROTATE_270;
+		data->tran_type = TRAN_NONE;
+		break;
+	default:
+		data->rot_type = TRAN_ROTATE_0;
+		data->tran_type = TRAN_NONE;
+		break;
+	}
+#endif /* HAVE_LIBEXIF */
+}
+
+
+static void
+update_from_exif_data (DialogData *data)
+{
+#ifdef HAVE_LIBEXIF
+	gboolean   from_exif;
+	GdkPixbuf *src_pixbuf, *dest_pixbuf;
+	
+	from_exif = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton));
+	gtk_widget_set_sensitive (data->j_button_box, !from_exif);
+
+	if (! from_exif)
+		return;
+	
+	update_rotation_from_exif_data (data, data->current_image);
+
+	src_pixbuf = data->original_preview;
+	
+	if (src_pixbuf == NULL) 
+		return;
+
+	switch (data->rot_type) {
+	case TRAN_ROTATE_90:
+		dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, FALSE);
+		break;
+	case TRAN_ROTATE_180:
+		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, TRUE);
+		break;
+	case TRAN_ROTATE_270:
+		dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, TRUE);
+		break;
+	default:
+		dest_pixbuf = src_pixbuf;
+		g_object_ref (dest_pixbuf);
+			break;
+	}
+
+	src_pixbuf = dest_pixbuf;
+
+	switch (data->tran_type) {
+	case TRAN_MIRROR:
+		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
+		break;
+	case TRAN_FLIP:
+		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, FALSE, TRUE);
+		break;
+	default:
+		dest_pixbuf = src_pixbuf;
+		g_object_ref (dest_pixbuf);
+			break;
+	}
+	
+	g_object_unref (src_pixbuf);
+
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
+		
+	g_object_unref (dest_pixbuf);
+
+#endif /* HAVE_LIBEXIF */
+}
+
+
 static void 
 image_loader_done_cb (ImageLoader  *il,
 		      DialogData   *data)
@@ -147,12 +264,15 @@ image_loader_done_cb (ImageLoader  *il,
 
 	pixbuf = image_loader_get_pixbuf (il);
 	if (pixbuf != NULL) {
-		GdkPixbuf *scaled = _gdk_pixbuf_scale_keep_aspect_ratio (pixbuf, PREVIEW_SIZE, PREVIEW_SIZE);
-		gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), scaled);
-		g_object_unref (scaled);
+		if (data->original_preview != NULL)
+			g_object_unref (data->original_preview);
+		data->original_preview = _gdk_pixbuf_scale_keep_aspect_ratio (pixbuf, PREVIEW_SIZE, PREVIEW_SIZE);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), data->original_preview);
 
-		gtk_widget_set_sensitive (data->j_button_box, TRUE);
+		gtk_widget_set_sensitive (data->j_button_vbox, TRUE);
 		gtk_widget_set_sensitive (data->j_revert_button, TRUE);
+
+		update_from_exif_data (data);
 	}
 }
 
@@ -161,7 +281,7 @@ static void
 image_loader_error_cb (ImageLoader  *il,
 		       DialogData   *data)
 {
-	gtk_widget_set_sensitive (data->j_button_box, TRUE);
+	gtk_widget_set_sensitive (data->j_button_vbox, TRUE);
 	gtk_widget_set_sensitive (data->j_revert_button, TRUE);
 }
 
@@ -176,7 +296,7 @@ load_current_image (DialogData *data)
 		return;
 	}
 	
-	gtk_widget_set_sensitive (data->j_button_box, FALSE);
+	gtk_widget_set_sensitive (data->j_button_vbox, FALSE);
 	gtk_widget_set_sensitive (data->j_revert_button, FALSE);
 
 	fd = data->current_image->data;
@@ -240,8 +360,33 @@ swap_fields (ExifContent *content,
 }
 
 
+static ExifShort
+get_next_value_rotation_90 (int value)
+{
+	static ExifShort new_value [8] = {8, 7, 6, 5, 2, 1, 4, 3};
+	return new_value[value - 1];
+}
+
+
+static ExifShort
+get_next_value_mirror (int value)
+{
+	static ExifShort new_value [8] = {2, 1, 4, 3, 8, 7, 6, 5};
+	return new_value[value - 1];
+}
+
+
+static ExifShort
+get_next_value_flip (int value)
+{
+	static ExifShort new_value [8] = {4, 3, 2, 1, 6, 5, 8, 7};
+	return new_value[value - 1];
+}
+
+
 static void
-swap_xy_exif_fields (const char *filename)
+swap_xy_exif_fields (const char *filename,
+		     DialogData *data)
 {
 	JPEGData     *jdata;
 	ExifData     *edata;
@@ -287,6 +432,81 @@ swap_xy_exif_fields (const char *filename)
 }
 
 
+static void
+update_orientation_field (const char *filename,
+			  DialogData *data)
+{
+	JPEGData     *jdata;
+	ExifData     *edata;
+	unsigned int  i;
+	gboolean      orientation_changed = FALSE;
+
+	jdata = jpeg_data_new_from_file (filename);
+	if (jdata == NULL)
+		return;
+
+	edata = jpeg_data_get_exif_data (jdata);
+	if (edata == NULL) {
+		jpeg_data_unref (jdata);
+		return;
+	}
+
+	for (i = 0; i < EXIF_IFD_COUNT; i++) {
+		ExifContent *content = edata->ifd[i];
+		ExifEntry   *entry;
+
+		if ((content == NULL) || (content->count == 0)) 
+			continue;
+
+		entry = exif_content_get_entry (content, EXIF_TAG_ORIENTATION);
+		if (!orientation_changed && (entry != NULL)) {
+			ExifByteOrder byte_order;
+			ExifShort     value;
+
+			byte_order = exif_data_get_byte_order (edata);
+			value = exif_get_short (entry->data, byte_order);
+
+			switch (data->rot_type) {
+			case TRAN_ROTATE_90:
+				value = get_next_value_rotation_90 (value);
+				break;
+			case TRAN_ROTATE_180:
+				value = get_next_value_rotation_90 (value);
+				value = get_next_value_rotation_90 (value);
+				break;
+			case TRAN_ROTATE_270:
+				value = get_next_value_rotation_90 (value);
+				value = get_next_value_rotation_90 (value);
+				value = get_next_value_rotation_90 (value);
+				break;
+			default:
+				break;
+			}
+			
+			switch (data->tran_type) {
+			case TRAN_MIRROR:
+				value = get_next_value_mirror (value);
+				break;
+			case TRAN_FLIP:
+				value = get_next_value_flip (value);
+				break;
+			default:
+				break;
+			}
+
+			exif_set_short (entry->data, byte_order, value);
+
+			orientation_changed = TRUE;
+		}
+	}
+
+	jpeg_data_save_file (jdata, filename);
+
+	exif_data_unref (edata);
+	jpeg_data_unref (jdata);
+}
+
+
 #endif
 
 
@@ -449,7 +669,8 @@ apply_tranformation_jpeg (DialogData *data,
 	} else {
 #ifdef HAVE_LIBEXIF
 		if ((rot_type == TRAN_ROTATE_90) || (rot_type == TRAN_ROTATE_270))
-			swap_xy_exif_fields (fd->path);
+			swap_xy_exif_fields (fd->path, data);
+		update_orientation_field (fd->path, data);
 #endif
 
 		data->files_changed_list = g_list_prepend (data->files_changed_list, 
@@ -534,12 +755,39 @@ static void
 apply_transformation (DialogData *data,
 		      GList      *current_image)
 {
-	FileData *fd = current_image->data;
+	FileData    *fd = current_image->data;
+	char        *dir;
+	struct stat  buf;
+
+	/* Check directory permissions. */
+
+	dir = remove_level_from_path (fd->path);
+	if (! check_permissions (dir, R_OK | W_OK | X_OK)) {
+		char *utf8_path;
+		utf8_path = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
+		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
+				       _("You don't have the right permissions to create images in the folder \"%s\""),
+				       utf8_path);
+		g_free (utf8_path);
+		g_free (dir);
+
+		return;
+	} 
+	g_free (dir);
+
+	/**/
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton)))
+		update_rotation_from_exif_data (data, current_image);
+
+	stat (fd->path, &buf);
 
 	if (image_is_jpeg (fd->path)) 
 		apply_tranformation_jpeg (data, current_image);
 	else 
 		apply_transformation_generic (data, current_image);
+
+	chmod (fd->path, buf.st_mode);
 }
 
 
@@ -619,7 +867,13 @@ static void
 revert_clicked (GtkWidget  *button, 
 		DialogData *data)
 {
-	load_current_image (data);
+	data->rot_type = TRAN_ROTATE_0;
+	data->tran_type = TRAN_NONE;
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton), FALSE);
+
+	if (data->original_preview != NULL)
+		gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), data->original_preview);
 }
 
 
@@ -773,6 +1027,14 @@ help_cb (GtkWidget  *widget,
 }
 
 
+static void
+from_exif_toggled_cb (GtkToggleButton *button,
+		      DialogData      *data)
+{
+	update_from_exif_data (data);
+}
+
+
 void
 dlg_jpegtran (GThumbWindow *window)
 {
@@ -817,6 +1079,7 @@ dlg_jpegtran (GThumbWindow *window)
 	data->dialog = glade_xml_get_widget (data->gui, "jpeg_rotate_dialog");
 	data->j_apply_to_all_checkbutton = glade_xml_get_widget (data->gui, "j_apply_to_all_checkbutton");
 	data->j_button_box = glade_xml_get_widget (data->gui, "j_button_box");
+	data->j_button_vbox = glade_xml_get_widget (data->gui, "j_button_vbox");
 	data->j_revert_button = glade_xml_get_widget (data->gui, "j_revert_button");
 	data->j_preview_image = glade_xml_get_widget (data->gui, "j_preview_image");
 
@@ -827,6 +1090,8 @@ dlg_jpegtran (GThumbWindow *window)
 	j_v_flip_button = glade_xml_get_widget (data->gui, "j_v_flip_button");
 	j_h_flip_button = glade_xml_get_widget (data->gui, "j_h_flip_button");
 
+	data->j_from_exif_checkbutton = glade_xml_get_widget (data->gui, "j_from_exif_checkbutton");
+
 	j_help_button = glade_xml_get_widget (data->gui, "j_help_button");
 	j_cancel_button = glade_xml_get_widget (data->gui, "j_cancel_button");
 	j_ok_button = glade_xml_get_widget (data->gui, "j_ok_button");
@@ -844,6 +1109,10 @@ dlg_jpegtran (GThumbWindow *window)
 	gtk_widget_set_sensitive (data->j_apply_to_all_checkbutton,
 				  data->file_list->next != NULL);
 
+#ifndef HAVE_LIBEXIF
+	gtk_widget_set_sensitive (data->j_from_exif_checkbutton, FALSE);
+#endif /* ! HAVE_LIBEXIF */
+
 	/* Set the signals handlers. */
 	
 	g_signal_connect (G_OBJECT (data->dialog),
@@ -883,6 +1152,10 @@ dlg_jpegtran (GThumbWindow *window)
 			  "clicked",
 			  G_CALLBACK (flip_clicked),
 			  data);
+	g_signal_connect (G_OBJECT (data->j_from_exif_checkbutton),
+			  "toggled",
+			  G_CALLBACK (from_exif_toggled_cb),
+			  data);
 
 	data->loader = (ImageLoader*)image_loader_new (NULL, FALSE);
 
